/**
* Name: CoreLiBIM
* Author: Mog
* Description: 
* Tags: Tag1, Tag2, TagN
*/

model CoreLiBIM

import "Occ_Li_BIM_2017.gaml"

// BEGINNING OF THE AGENT WORLD ***************************************************************************

global
{
	
	// Data for the Visualization
	file<geometry> ifcfile <- ifc_file("../includes/Projet AUGC 2017 V3.ifc");
	geometry shape <- envelope(ifcfile) + 2;
	bool draw_toit <- false parameter:false;
	file clock_normal     const: true <- image_file("../images/clock.png");
	file clock_big_hand   const: true <- image_file("../images/big_hand.png");
	file clock_small_hand const: true <- image_file("../images/small_hand.png");
	
	//Data import from Environement (Weather, etc.)
	string weather_file_name <-  "../includes/Meteo.csv" ;
	file weather_file <- csv_file(weather_file_name,";",true);
	
	//Data from devices
	string devices_file_name <-  "../includes/devices_data.csv" ;
	file device_file <- csv_file(devices_file_name,";",true);
	
	//Data from the Inhabitants
	file inhabitant_file <- csv_file("../includes/occ_data.csv",";", true);

	string result_file <- "../includes/results.csv";
	//string result_light <- "../includes/light_consumption.csv";
	string result_NRJ <- "../includes/NRJ_consumption.csv";
	string result_occupancy <- "../includes/occupancy.csv";
	
	//for communication between models
	//string TM_file <-  "../includes/Physical_Model/Thermal/TM.jar" ;
	string thermal_fileC <- "../models/thermalC.csv";
	string thermal_fileV <- "../models/thermalV.csv";
	file thermal_fileR <- csv_file("../models/Resul_Th.csv",";");
	matrix RMT;
	bool ext_TM<-false;  //True : on utilise le modèle thermiqe en java, false : on utilise le modèle dans gama
	
	
	//Environement state (Outside Temperature and Humidity)
	float temperature;  //température extérieure
	float humidity; 	//humidité extérieure
	matrix weather_data;	//matrice avec temp et hum selon l'heure et la date
	float temperature_ref;	//température utilisée pour le confort d'apres les températures sur une plage donnée
			
	//Agenda
	matrix working_day;		//liste jour de travail
	bool work_today<-true;	//est-ce un jour ou il y a travail ?
	file work_days_file <- csv_file("../includes/jours_ouvres2017.csv",";", true);	
	matrix sun_hours;	//heure de lever et coucher de soleil selon le jour
	bool night<-true;	//est ce qu'il fait nuit <,
	file sun_hours_file <- csv_file("../includes/calendrier_solaire.csv",";", true);
	
	//G_xxx sont des vecteurs (variable globale) reprenant les valeurs actuelles des indicateurs de chaque occupant
	list<float> G_energy; //from 0:exhausted to 1:well-rested
	list<float> G_fullness; //from 0:starving  to 1:full
	list<float> G_cleanliness; //from 0:cleanliness to1:clean
	list<float> G_toilet; //de 0:urgent to 1:perfect
	list<float> G_wash_clothes; //de 0:urgent to 1:perfect
	list<float> G_fresh_air; //de 0:urgent to 1:perfect
	list<float> G_house_cleanliness; //de 0:urgent to 1:perfect
	list<float> G_smoke; //de 0:urgent to 1:perfect


	
//BEGINNING OF THE INITIALISATION -------------------------------------------
	init
	{
		//paramètres de temps pouvant être modifés
		step <- 5 # mn; //step time of the simulation
		starting_date <- date([2017, 1, 1, 0, 0, 0]); //date of the beginning of the simulation
		
		
		//lecture of the data from the IFC file
		loop g over: ifcfile
		{
			container my_size <- [g.height, g.depth, g.width] sort_by (each);
			float S <- my_size[1]*my_size[2];
			g <- g translated_by {2,2};
			string t <- g get "type";
			switch t
			{
				match "IfcWallStandardCase"
				{						
					create wall with: [shape::g, name:: g get "name", U_wall:: float(g get "ThermalTransmittance"), S_wall:: float(S), outside::bool(g get "IsExternal")];		
				}

				match "IfcSlab"
				{
					create slab with: [shape::g, name:: g get "name", U_slab:: float(g get "ThermalTransmittance"), S_slab:: float(g.area), outside::bool(g get "IsExternal") ];
				}

				match "IfcSpace"
				{
					create room with: [shape::g scaled_by 0.9, name:: g get "Reference", type:: (string(g get "Reference") split_with " ")[0]] ;
					
				}

				match "IfcWindow"
				{
				create window with: [shape::g, name:: g get "name", U_window:: float(g get "ThermalTransmittance"), S_window:: float(S),outside::bool(g get "IsExternal")];
				}

				match "IfcDoor"
				{
					create door with: [shape::g, name:: g get "name", U_door:: float(g get "ThermalTransmittance"), S_door:: float(S),outside::bool(g get "IsExternal")];
				}

			}

		} 

		//création de la maison et du foyer
		create building;
		create household;	
		
		//calculation of the U of the building
		ask building{	
			ask wall {
				if outside {myself.Uw_without_windows<-myself.Uw_without_windows+self.S_wall*self.U_wall;}
			}
			ask slab {
				if outside {myself.Uw_without_windows<-myself.Uw_without_windows+self.S_slab*self.U_slab;}
			}
			ask door {
				if outside {myself.Uw_without_windows<-myself.Uw_without_windows+self.S_door*self.U_door;}
			}
			//on calcule le Uw de la partie qui est fixe durant la simulation (Uw_without_windows) et on ajoute ensuite la partie due au fenetre (variable) en supposant au départ les fenetres fermées
			Uw_tot<-Uw_without_windows;
			ask window {
				if outside {myself.Uw_tot<-myself.Uw_tot+self.S_window*self.U_window;}
				my_room<-room closest_to self;
			}
		}
	
	 
		//lecture of the external file (weather, agenda, sun)
		weather_data <- matrix(weather_file);	//convert the weathetr file into a matrix
		do define_weather;		//calcule la métoé actuelle
		
		working_day<- matrix(work_days_file);	
		sun_hours <- matrix(sun_hours_file);	

		//creation de l'extérieur
		create room {
			name<-"outside";
			type <-name;
			shape <- ((world.shape scaled_by 0.9) - (union(room) + 0.2)) ;
		}
		
		//ajout de frioriture (meubles) dans les pièces
		ask room {
			shape_free <- copy(shape);
			if (type = "Cuisine") {
				create kitchen_obj with:[location::location] {myself.shape_free <- myself.shape_free - shape_emprise;}	
				
			} else if (type = "Chambre") {
				create bed_obj with:[location::location] {myself.shape_free <- myself.shape_free - shape_emprise;}	
			
			} else if (type = "WC") {
				create toilet_obj with:[location::location]{myself.shape_free <- myself.shape_free - shape_emprise;}
			
			} else if (type = "Sejour") {
				create sopha_obj with:[location::location]{myself.shape_free <- myself.shape_free - shape_emprise;}
				
			} else if (type = "Salle") {
				create bath_obj with:[location::location]{myself.shape_free <- myself.shape_free - shape_emprise;}
			
				}
			ask wall at_distance 0.5{
				myself.shape_free <- myself.shape_free - (self + 0.4);
			}
		

		
		}
		
		create device from:device_file with:
				[use::string(get("Use")), 
					loc::string(get("My_location")), 
					category::int(get("Category")), 
					duration::float(get("Duration")), 
					power_mode1::float(get("Power_mode1")), 
					power_mode2::float(get("Power_mode2")), 
					power_mode2eco::float(get("Power_mode2_eco")), 
					my_activity::int(get("My_activity")),	
					mode_regul::int(get("Regul_mode"))
		];	
		
		ask device {my_location<- one_of(room where (each.name=loc));	}
		
		
		//Creation of the Inhabitants (lecture dans le fichier)
		create occupant from:inhabitant_file with:
				[name::string(get("Name")), 
			gender::int(get("Gender")),
			adult_child::int(get("Adult")),
			job::bool(get("Job")),
			smoker::bool(get("Smoke")),
			ideal_hour_to_sleep ::float(get("Hsleep")),
			ideal_hour_for_diner ::float(get("Hdiner")),
			ideal_hour_for_lunch ::float(get("Hlunch")),
			ideal_hour_for_breakfast ::float(get("Hbreakfast")),
			leave_home_to_job ::float(get("Hjob")),
			return_home_after_job  ::float(get("HjobEnd")),
			sensitive_to_cold::float(get("Sensitivecold")),
			sensitive_to_warm::float(get("Sensitivewarm")),
			h_hungryness  ::float(get("NbHhunger"))# h,
			h_tiredness  ::float(get("NbHtire"))# h,
			h_dirtyness ::float(get("NbHdirty"))# h,
			h_need_to_toilet  ::float(get("NbHWC"))# h,
			h_lack_smoking ::float(get("NbHsmoke"))#h,
			h_smoke::float(get("ANbHsmoke"))#mn,
			h_wash_clothes::float(get("ANbHwash"))#mn,
			h_dirty_clothes ::float(get("NbHwash"))#h,	
			h_eat  ::float(get("ANbHhunger"))# mn,
			h_cook  ::float(get("ANbHcook"))# mn,
			h_sleep ::float(get("ANbHtire"))# h,
			h_shower::float(get("ANbHdirty"))# mn,
			h_poop::float(get("ANbHWC"))# mn, 
			h_dirty_house::float(get("NbHclean"))#h,
			h_clean_house::float(get("ANbHclean"))#mn,
			h_need_fresh_air::float(get("NbHfreshair"))#h,
			h_take_fresh_air::float(get("ANbHfreshair"))#mn,
			green_conscious::int(get("GreenConscious")),
			building_knowledge::int(get("BuildingKnowledge")),
			individualism::int(get("Individualism")), 
			wealth::int(get("Wealth"))
			];	
		
		
		//on met tous les habiants dans	le même foyer		
		ask household {	ask occupant {add self to:myself.member;}}
		
		int i<-1;
		ask occupant {
			number<-i;
			i<-i+1;
			use_social_architecture<-true;		//on utilise les liens sociaux de gama
			last_temp_modif<-current_date; 	//on défnit la dernière modif de la température: itialiser à la date actuelle
			last_temp_modif_clo<-current_date; 	//on défnit la dernière fois où on achnagé ses vétements: itialiser à la date actuelle
			delta_temp_cold <- 5*(1-sensitive_to_cold);
			delta_temp_warm <- 5*(1-sensitive_to_warm);
			ideal_temp<-19-delta_temp_cold+delta_temp_warm;
			

			//définit les membres de sa famille
			ask household {
				if self.member contains myself {
					loop occ over:self.member {
						if (occ!=myself and !(myself.my_family contains occ)) {
							add occ to:myself.my_family;	
						}
					}
				}
			}
			
			//on définit le profil type de chaque occupant
			if (wealth=1) {
				if building_knowledge=1 or building_knowledge=0 {Profile <- 1;}
				if building_knowledge=2  {Profile <- 2;}
			}
			
			if (wealth=2) {
				if green_conscious=0 {
					if building_knowledge=0 {Profile <- 3;	}
					if building_knowledge=1 {Profile <- 4;	}
					if building_knowledge=2 {Profile <- 5;	}
				}
				
				if green_conscious=1 {
					if individualism=0 {
						if building_knowledge=0 {Profile <- 6;	}
						if building_knowledge=1 {Profile <- 7;	}
						if building_knowledge=2 {Profile <- 8;	}
					}
					
					if individualism=1  {
						if building_knowledge=0 {Profile <- 9;	}
						if building_knowledge=1 {Profile <- 10;	}
						if building_knowledge=2 {Profile <- 11;	}
					}
				}
				
				if green_conscious=2 {
					if individualism=0 {
						if building_knowledge=0 {Profile <- 12;	}
						if building_knowledge=1 {Profile <- 13;	}
						if building_knowledge=2 {Profile <- 14; }
					}
					if individualism=1 {
						if building_knowledge=0 {Profile <- 1;	}
						if building_knowledge=1 {Profile <- 15;	}
						if building_knowledge=2 {Profile <- 16;	}
					}
				}				
			}
			
			if wealth>=3 {
				if green_conscious=0 {
					if building_knowledge=0 {Profile <- 3;	}
					if building_knowledge=1 {Profile <- 4;	}
					if building_knowledge=2 {Profile <- 17;	}
				}
				
				if green_conscious=1 {
					if individualism=1 {
						if building_knowledge=0 {Profile <- 18;	}
						if building_knowledge=1 {Profile <- 19;	}
						if building_knowledge=2 {Profile <- 20;	}
					}
					if individualism=0 {
						if building_knowledge=0 {Profile <- 22;	}
						if building_knowledge=1 {Profile <- 23;	}
						if building_knowledge=2 {Profile <- 24;	}
					}
				}
				
				if green_conscious=2 {
					if individualism=1 {
						if building_knowledge=0 {Profile <- 25;	}
						if building_knowledge=1 {Profile <- 26;	}
						if building_knowledge=2 {Profile <- 27;	}
					}
					if individualism=0  {
						if building_knowledge=0 {Profile <- 28;	}
						if building_knowledge=1 {Profile <- 29;	}
						if building_knowledge=2 {Profile <- 30;	}
					}
				}
			}
			
			if Profile=1 or Profile=3 or Profile=4 or Profile=5 or Profile=9 or Profile=10 or Profile=11 or Profile=17 or Profile=18 or Profile=19 or Profile=20 {
			//	delta_date_temp_modif<-30.0 #mn; 
				thermal_effort<-0.0;
			}
			else {
			//	delta_date_temp_modif<-40.0 #mn; 
				thermal_effort<-5.0;  //delata de temp à partir de laquelle j'allume vraiment le chauffage en hiver
			}



			if Profile>18 {
				ask device where (each.use="Light") {
					power_mode2<-power_mode2eco;
				}
			}
			
				if Profile=12 or Profile=13 or Profile=14 or Profile=22 or Profile=23 or Profile=24 or Profile>27 {
				ask device where (each.use!="Light") {
					power_mode2<-power_mode2eco;
				}
			}
			
			//on met comme valeur de priorité des intentions, les valeurs par défut au début (peut évoluer)
			P_work <- Def_P_work;
			P_eat <- Def_P_eat;
			P_sleep <- Def_P_sleep;
			P_relax <- Def_P_relax;
			P_go_to_wc <- Def_P_go_to_wc;
			P_take_a_shower <- Def_P_take_a_shower;
			P_mod_temp<-Def_P_mod_temp;
			P_smoke <- Def_P_smoke;
			P_wash_clothes <- Def_P_wash_clothes;
			P_go_out <- Def_P_go_out;
			P_clean_house<- Def_P_clean_house;
			
			//adult_child=1 (enfant) ou 2 (adulte) / gender=1 (femme) ou 2 (homme); on choisit en fonction la représentation 3D
			if (adult_child=2 and gender=1) {
				my_file <- "../3Dmodel/female03/female03.obj";
				mycolor <- # khaki;
				mysize <- 1.6;
				translation <- mysize/2.0;
				init_rotation <- 90::{-1,0,0};
				my_bedroom<- one_of(room where (each.name="Chambre 1 2"));
			}
				
			if (adult_child=2 and gender=2) {
				my_file <- "../3Dmodel/holdingMale01/holdingMale01.obj";
				mycolor <- # blue;
				mysize <- 1.6;
				translation <- mysize/2.0;
				init_rotation <- 90::{-1,0,0};
				my_bedroom<- one_of(room where (each.name="Chambre 1 2"));
			}
				
			if (adult_child=1 and gender=1) {
				my_file <- "../3Dmodel/girlStanding/girlStanding.obj";
				mycolor <- # khaki;
				mysize <- 1.2;
				translation <- mysize/2.0;
				init_rotation <- 90::{-1,0,0};
				Profile <- 6;
				my_bedroom<- one_of(room where (each.name="Chambre 2 1"));
			}
				
			if (adult_child=1 and gender=2) {
				my_file <- "../3Dmodel/boyStanding/boyStanding.obj";
				mycolor <- # blue;
				mysize <- 1.2;
				translation <- mysize/2.0;
				init_rotation <- 90::{-1,0,0};
				Profile <- 6;
				my_bedroom<- one_of(room where (each.name="Chambre 2 1"));
			}
				
			//initialise au départ un désir de se reposer et met toutes les barres à 1 (full)
			do add_desire(relax);
			my_energy <- 1.0; 
			my_fullness <- 1.0; 
			my_cleanliness <- 1.0;
			my_need_to_toilet <- 1.0;
			my_comfort<-1.0;
			my_will_to_wash_clothes<-1.0;
			my_need_of_fresh_air<-1.0;
			house_cleanliness<-1.0;
			my_need_to_smoke<-1.0;
			
			
			//ajoute les données de base dans les vecteurs, pour affichcage du radar et graphique
			add my_energy to: G_energy;
			add my_fullness to: G_fullness;
			add my_cleanliness to: G_cleanliness;
			add my_need_to_toilet to: G_toilet;
			add my_will_to_wash_clothes to: G_wash_clothes; //de 0:urgent to 1:perfect
			add my_need_of_fresh_air to: G_fresh_air; //de 0:urgent to 1:perfect
			add house_cleanliness to: G_house_cleanliness; //de 0:urgent to 1:perfect
			add my_need_to_smoke to: G_smoke;
			add activity to: building[0].activity_occupant;
			add P_occ to:building[0].P_occupant;
		
			//creation lien social (tirer au sort pour l'instant)
			loop fam_mem over:my_family {
					if adult_child=2 { //si je suis un adulte
					if fam_mem.adult_child=2 { //mes liens sociaux avec les autres adultes de la famille
						do add_social_link(new_social_link(fam_mem) set_familiarity 1.0 set_liking (rnd(1100)/1000-0.1) set_dominance (rnd(1300)/1000-0.3) set_solidarity (rnd(1000)/1000));
					}
					else { //mes liens sociaux avec les enfants de la famille
						do add_social_link(new_social_link(fam_mem) set_familiarity 1.0 set_liking (rnd(1200)/1000-0.2) set_dominance (rnd(1900)/1000-0.9) set_solidarity (rnd(1500)/1000-0.5));
					}
				}
			
				else { // si je suis un enfant
					if fam_mem.adult_child=2 { //mes liens sociaux avec les autres adultes de la famille
						do add_social_link(new_social_link(fam_mem) set_familiarity 1.0 set_liking (rnd(1100)/1000-0.1) set_dominance (rnd(1000)/1000) set_solidarity (rnd(1000)/1000));
					}
					else { //mes liens sociaux avec les enfants de la famille
						do add_social_link(new_social_link(fam_mem) set_familiarity 1.0 set_liking (rnd(1000)/1000) set_dominance (rnd(1000)/1000) set_solidarity (rnd(1000)/1000));
					}				
				}
			}
		} //fin initialisation des occupants
		
		
		ask device where (each.category=1) {power_mode2<-power_mode2*duration/step;}
		ask device where (each.category=0) {switch_on<-2;}
		
		ask building {
			ask device where (each.use="Heater") {myself.type_regul<-self.mode_regul; }
		
		save[""] to:thermal_fileC type:"csv" ; //vide le fichier
		save[temp_chau,Rth_rad,Pmaxchaud, Cth_h, Cth_ch,step] to:thermal_fileC type:"csv" ;
		}
				
} // END OF THE INITIALIZATION  -------------------------------------------------------------------------------


	//Update at each step time the value of the global indicator 
	reflex eval	{
		int i <- 0;
		ask occupant {
			G_energy[i] <- self.my_energy; //from 0:exhausted to 1:well-rested
			G_fullness[i] <- self.my_fullness; //from 0:starving  to 1:full
			G_cleanliness[i] <- self.my_cleanliness; //from 0:cleanliness to 1:clean
			G_toilet[i] <- self.my_need_to_toilet; //de 0:urgent to 1:perfect
			G_wash_clothes[i] <- self.my_will_to_wash_clothes; //de 0:urgent to 1:perfect
			G_fresh_air[i] <- self.my_need_of_fresh_air; //de 0:urgent to 1:perfect
			G_house_cleanliness[i] <- self.house_cleanliness; //de 0:urgent to 1:perfect
			G_smoke[i] <- self.my_need_to_smoke; //de 0:urgent to 1:perfect
			i <- i + 1;
		
			avg_comfort<-avg_comfort+my_comfort;
			if self.activity=0 {my_activity_color <- rgb(0,0,0); }  //black=sleep
			if self.activity=1 {my_activity_color <- rgb(255,0,255); }  //pink:relax
			if self.activity=2 {my_activity_color <- rgb(0,191,255); } //blue: shower
			if self.activity=3 {my_activity_color <- rgb(205,133,63); } //marroun: WC
			if self.activity=4 {my_activity_color <- rgb(238,130,238); } //viloet: eat
			if self.activity=5 {my_activity_color <- rgb(147,112,216); }  //purple: cook
			if self.activity=6 {my_activity_color <- rgb(255,0,0); }  //red:work
			if self.activity=7 {my_activity_color <- rgb(119,136,153); }  //gray:smoke
			if self.activity=8 {my_activity_color <- rgb(72,209,204); } //turquoise:wash clothes
			if self.activity=9 {my_activity_color <- rgb(0,255,0); }  //green: go out
			if self.activity=10 {my_activity_color <- rgb(255,165,0); } //orange: clean house
			if self.activity=11 {my_activity_color <- rgb(255,255,0); } //yellow:discuss comfort
			if self.activity=12 {my_activity_color <- rgb(255,215,0); } //gold: discuss GO
			if self.activity=13 {my_activity_color <- rgb(85,107,47); }  //dark gree: modify temp
		}




//Gestion de la FIN DE LA SIMULATION (au bout d'une année simulée)
		if current_date.year>starting_date.year {
			//Fichier activité		
			save[""] to:result_file type:"csv" ; //vide le fichier
			//Activity 0:sleep, 1:relax, 2:shower, 3:wc,  4:eat, 5:cook, 6:work, 7:smoke, 8:wash cloth, 9:go out, 10:clean house, 11:discuss comfort, 12:discuss GO, 13: modify temp
			
			save["Name","sleep","relax", "shower", "wc",  "eat", "cook", "work", "smoke", "wash cloth", "go out", "clean house", "discuss comfort", "discuss GO", "modify temp","comfort"] to:result_file header:true type:"csv" ; //save les entetes
			//on sauve les données pour chaque habitant
			ask occupant {
				save[name,cycle_activity[0],cycle_activity[1],cycle_activity[2],cycle_activity[3],cycle_activity[4],cycle_activity[5],cycle_activity[6],cycle_activity[7],cycle_activity[8],cycle_activity[9],cycle_activity[10],cycle_activity[11],cycle_activity[12],cycle_activity[13],avg_comfort/cycle] to:result_file rewrite:false type:"csv";
			}
			
					
			//Fichier consommation
			save[""] to:result_NRJ type:"csv" ; 
			save["Device", "Type", "room", "NRJ"] to:result_NRJ header:true type:"csv"; //sauve les entetes
			//sauve les données liés à l'éclairage
			ask device {
				save [name, use, my_location.name, Total_NRJ/1000] to:result_NRJ rewrite:false type:"csv";
			}
			
			//Fichier présence des occupants par pièce
			save[""] to:result_occupancy type:"csv" ; //vide le fichier result_occupancy
			save["Room", "Nb of Hours in the room"] to:result_occupancy header:true type:"csv"; //sauve les entetes
			//sauve les données liés à l'éclairage
			ask room {
				save [name, count_occupancy] to:result_occupancy rewrite:false type:"csv";
			}

			write "End";	//affiche fin de simulation
			do halt;		//arrete simulation
		}
	}

	//Update the weather and the agenda	
	reflex update_weather_and_working_day {
		do define_weather;
		do define_working_day;
		do define_night;	
	}		
	
	//on cherche la température et l'humidité du pas de temps donné
	action define_weather {
		//loop on the matrix rows (skip the first header line)
		loop i from: 0 to: weather_data.rows {		
			if (float(current_date.month)=weather_data[1,i] and float(current_date.day)=weather_data[2,i] and float(current_date.hour+1)=weather_data[3,i]){
				temperature<-float(weather_data[4,i]);
				humidity<-float(weather_data[5,i]);
				temperature_ref<-((float(weather_data[4,i])+0.8*float(weather_data[4,i-1])+0.8*float(weather_data[4,i-1])+0.4*float(weather_data[4,i-2])+0.2*float(weather_data[4,i-3]))/2.4);
			}
		}	
	}
	
	//on cherche si c'est un jour où l'on travaille
	action define_working_day {
		work_today<-false;
		loop i from: 0 to:working_day.rows {			
			if (int(current_date.month)=int(working_day[1,i]) and int(current_date.day)=int(working_day[0,i])) {
					work_today<-true;
			}							
		}	
	}
	
	//on cherche s'il fait nuit
	action define_night {
		night <-true;
		loop i from: 0 to:sun_hours.rows {			
				if (int(current_date.month)=int(sun_hours[2,i]) and int(current_date.day)=int(sun_hours[1,i]) and int(current_date.year)=int(sun_hours[3,i]))and (int(current_date.hour) <= int(sun_hours[6,i]) and int(current_date.hour) >= int(sun_hours[4,i])) {
						night<-false;	
				}						
		}		
	}



}  //*********************************END OF THE AGENT WORLD*****************************************



//******************************BEGINNING OF THE AGENT HOUSE ****************************************
species building {
	float temp_int<-20.0;
	float Pwall;
	float Prad;
	float Pchaud;
	float temp_chau<-60.0;		//°C
	float Rth_rad<-0.02;		//°C / W 
	float Pmaxchaud<-5000.0;	//W
	float Cth_h<-6*10^6;		// J/°C 
	float Cth_ch<-8*10^5;		// J/°C 
	list<int> activity_occupant; //reprend les activités de chaque occupant
	list<float> P_occupant;//heat flow of each occupant
	bool rad_on<-true; //true si le chauffage est allumé
	float P_tot_occ update:sum(P_occupant); // chaleur dégagé par tous les occupants de la maison
	float temp_regul<-19.0;
	int type_regul<-0; 			//0: manuel, 1: manuel/auto, 2:auto
	bool regul_manual<-false; //False si  régulation auto, true si manuel
	float thermostat<-0.5;		//valeur du thermostat: 0 éteint à 1 allumé à pleine puissance
	
	//Transmission thermique
	float Uw_tot<-0.0;
	float Uw_without_windows<-0.0;
	
		
	
	reflex update_Uw {
		Uw_tot<-Uw_without_windows;
		ask window {
			if outside {
				if open {myself.Uw_tot<-myself.Uw_tot+self.S_window*240.0;}
				else {		
					if blind_down {
						myself.Uw_tot<-myself.Uw_tot+self.S_window*(self.U_window-self.U_blind);
				    }  else {myself.Uw_tot<-myself.Uw_tot+self.S_window*self.U_window;}
				    
				}
			}
			my_room<-room closest_to self;
		}
	}
	
	//Update the Activity and the Heat Production of occupant
	reflex update_activity{
		int i<-0;
		loop occ over:occupant {
			activity_occupant[i]<-occ.activity;
			P_occupant[i]<-occ.P_occ;
			i<-i+1;
		}
	}
	
	

	action regul_thermostat {   //thermostat calculé automatiquement pour avoir Tint=Tidéale
		float Pradth;
		float temp_chauth;
		float Pchaudth;
		Pwall<-(temp_int-temperature)*Uw_tot;		
		Pradth<-Pwall-P_tot_occ+(temp_regul-temp_int)*Cth_ch/step;
		temp_chauth<-Pradth*Rth_rad+temp_int;
		Pchaudth<-(temp_chauth-temp_chau)*Cth_ch/step+Pradth;
		thermostat<-max([0,min([1,Pchaudth/Pmaxchaud])]);
	}

	reflex update_temp_int {  
	if regul_manual=false {do regul_thermostat;}
	if rad_on=false {thermostat<-0.0;}
	if !ext_TM {
		Pchaud<-thermostat*Pmaxchaud;
		ask device where (each.use="Heater") {power_mode2<-myself.Pchaud; }
		Prad<-max([0,(temp_chau-temp_int)/Rth_rad]);
		Pwall<-(temp_int-temperature)*Uw_tot;		
		temp_chau<-temp_chau+(Pchaud-Prad)/Cth_ch*step;
		temp_int<-temp_int+(Prad-Pwall+P_tot_occ)/Cth_h*step;
	
	} 
	else {
		save[""] to:thermal_fileV type:"csv" ; //vide le fichier
		save[temp_int,thermostat,temperature, Uw_tot, P_tot_occ] to:thermal_fileV type:"csv" ;
		string tmp <-command("java -jar \"TM.jar\"");
		thermal_fileR <- csv_file("../models/Resul_Th.csv",";");
		RMT<- matrix(thermal_fileR);
		temp_int<-float(RMT[0,0]);
		
		}
	}
	

	reflex compute_energy_consumption {
		ask occupant {
			if my_previous_activity!=activity {
				ask device where (each.my_activity=my_previous_activity and each.category>0) {
				if (myself.Profile=1 or myself.Profile=2 or myself.Profile=12 or myself.Profile=13 or myself.Profile=14 or myself.Profile=15 or myself.Profile=16 or myself.Profile>24) {
					switch_on<-0;
				}
				else  {	switch_on<-1;}
				
				}
			}
			ask device where (each.use="Light") {switch_on<-0;}
			ask device where (each.my_activity=activity and each.category>0) {switch_on<-2;	}
			if (night and at_home and get_current_intention() != sleep) {ask device where (each.my_location=my_location and each.use="Light") {switch_on<-2;}}				
		}
	}
}

//******************************BEGINNING OF THE AGENT HOUSEHOLD ****************************************
species household {
	//ne sert à rien pour l'instant, mais ça va servir pour le simmeubles collectifs
	list<occupant> member;
}


// ******************************BEGINNING OF THE AGENTS OBJECTS *************************************
species kitchen_obj {
	geometry shape_emprise <- box(({2.5,0.5,1})) at_location (location +{0,0.6,0.4});
	aspect default {
		draw obj_file("../3Dmodel/Kitchen/Kitchen.obj", 90::{-1,0,0}) at: location +{0,0,0.4}size: {2.5,2,2}  ;
	}
}

species bed_obj {
	geometry shape_emprise <- box(({2.0,2,2})) at_location (location +{-1,-1,0.4});
	aspect default {
		draw obj_file("../3Dmodel/bed1_obj/bed1.obj", 90::{-1,0,0}) rotate:90::{0,0,-1} at: location +{-1,-1,0.4}size: {2.0,2.0,1}  ;
	}
}

species sopha_obj {
	geometry shape_emprise <- box(({3,3,2})) at_location (location +{1,-2.0,0.4});
	aspect default {
		draw obj_file("../3Dmodel/sofa4/sofa4.obj", 90::{-1,0,0})  at: location +{1,-2.0,0.4}size: {2.0,3.0,2}  ;
	}
}

species toilet_obj {
	geometry shape_emprise <- box(({0.5,0.5,2})) at_location (location + {0,0.4,0.4});
	aspect default {
		draw obj_file("../3Dmodel/water/water.obj", 90::{-1,0,0})  rotate: 180::{0,0,1}at: location +{0,0.4,0.4}size: {0.5,0.5,0.5}  ;
	}
}

species bath_obj {
	geometry shape_emprise <- box(({2,1,1})) at_location (location +{1,1.0,1.3});
	aspect default {
		draw box(1.8,0.5,0.1) color: #lightblue at: location +{1,1.0,0.3};
		draw obj_file("../3Dmodel/bath_L/bath_L.obj", 90::{-1,0,0}) rotate: 180::{0,0,1} at: location +{1,1.0,1.3}size: {2,2,2}  ;
	}
}

// ******************************BEGINNING OF THE AGENTS WALLS *************************************
species wall {
	float U_wall;
	float S_wall;
	bool outside;
	aspect default {
		draw shape texture: "../textures/mur.jpeg";
	}
}


// ******************************BEGINNING OF THE AGENTS WINDOWS *************************************
species window {
	room my_room;
	float U_window; //propriété thermique de la fenêtre
	float U_blind<-0.22; //propriété thermique du volet W/m2/K
	float S_window; //proriété géométrique de la fenêtre
	bool open <- false; //état de la fenêtre : ouverte ou fermée
	bool outside; //propriété de la fenêtre : si elle donne sur l'extérieur
	bool blind_down<-false; //état du volet : descendu ou monté
	
	aspect default	{
		draw shape color: #aquamarine;
	}
	
	reflex update_blinds {
	//stores du salon + cuisine baissés en été pendant la jourée pour se protéger de la chaleur et de la luminosité
		blind_down<- false;
		ask occupant { 
		
		if Profile=2 or Profile=5 or  Profile=8 or Profile=11 or  Profile=14 or Profile=16 or  Profile=17 or Profile=20 or  
		Profile=24 or Profile=27 or  Profile=30  {
	//	if (humidity < 50) and (temperature > 28) and (night=false) { 
		if (temperature > 27 and !night) { 
		myself.blind_down<- true;
		//	if (myself.my_room="Cuisine") or (myself.my_room="Sejour") {
		//		myself.blind_down<- true;
		//	}
		}
		
			if (temperature <15) and (night) { //stores du salon+cuisine baissés en hiver pendant la nuit pour se protéger du froid
		myself.blind_down<- true;
		//	if (myself.my_room="Cuisine") or (myself.my_room="Sejour") {
		//		myself.blind_down<- true;
		//	}
		}
		
		}
		
		//stores de la chambre baissés pendant que les occupants dorment
			if (is_sleeping)  {
				if (myself.my_room="Chambre") {
					myself.blind_down<- true;
				}
			}
		}
		
	
	}
}

// ******************************BEGINNING OF THE AGENTS DOORS *************************************
species door {
	float U_door;
	float S_door;
	bool outside;
	aspect default	{
		draw shape texture: "../textures/bois.jpg";
	}
}

// ******************************BEGINNING OF THE AGENTS SLAB *************************************
species slab {
	float U_slab;
	float S_slab;
	bool outside;
	aspect default 	{
		if (location.z > 0) {
			if (draw_toit) {
				draw shape texture: "../textures/tuile.jpeg";	
			}
		} 
		else {
			draw shape texture: "../textures/parquet.jpg";
		}
	}
}


// ******************************BEGINNING OF THE AGENTS ROOMS *************************************
species room {
	string type;
	geometry shape_free;
	int count_occupancy;
	float temperature_upper;
	float temperature_lower;
	float temperature_neutral;
	

	
//COMFORT (Determination of the comfort temperature depending on the room)
	reflex define_temp {
		if (type ="Chambre"){
			if (temperature_ref<0){
				temperature_neutral<-16.0;
			}
			if (0<temperature_ref) and (temperature_ref<12.6) {
				temperature_neutral<-0.23*temperature_ref+16;
			}
			if (12.6<temperature_ref) and (temperature_ref<21.8) {
				temperature_neutral<-0.77*temperature_ref+9.18;
			}
			if (21.8<temperature_ref) {
				temperature_neutral<-26.0;
			}
		}
		
		if (type= "Salle"){
			if (temperature_ref<11){
				temperature_neutral<-0.112*temperature_ref+22.65;
			}
			if (11<=temperature_ref) {
				temperature_neutral<-0.306*temperature_ref+20.32;
			}
			
		}
		
		else {
			if (temperature_ref<12.5){
				temperature_neutral<-0.06*temperature_ref+20.4;
			}
			if (12.5<=temperature_ref) {
				temperature_neutral<-0.36*temperature_ref+16.63;
			}
		}
	temperature_upper<-temperature_neutral+5*0.7;
	temperature_lower<-temperature_neutral-5*(1-0.7);
	}
	
	aspect default	{
		draw shape.contour  color: #black;
		draw name font: font("Helvetica", 18, #bold) at: location - {0.8,0,0} color: #black;
	}
}	

// ******************************BEGINNING OF THE AGENTS DEVICE *************************************
species device {
	string use; //TV, refrigerator....
	int category;  //0: independant of occupant, 1:start by occupant, 2:start and swithoff by occupant
	int switch_on;  //0: switch off, 1: saving power mode, 2:switch on
	float power_mode2; //puissance en mode swith on
	float power_mode2eco; //puissance en mode swith on
	float power_mode1<-0.0; //puissance en mode veille
	room my_location;
	string loc;
	int my_activity;  //used for which activity
	float NRJ;
	float Total_NRJ;
	float duration; //pour mode 1)
	int mode_regul; //pour heater notamment
	
	
	reflex energy_consumption {
		if switch_on=0 {NRJ<-0.0;}
		if switch_on=1 {NRJ<-power_mode1*step/3600;}
		if switch_on=2 {NRJ<-power_mode2*step/3600;}
		Total_NRJ<-Total_NRJ+NRJ;
	}
	
	}

//**************************************BEGINNING OF THE EXPERIMENTS **********************************
experiment Mode3D type: gui {
	output	{
		display view background: (night=false) ? #lightgray : #black type: opengl synchronized: true 	{
			image "../textures/herbe.jpg" transparency: 0.3 refresh: false ;
			species wall refresh: false;
			species slab ;
			species room refresh: false;
			species kitchen_obj refresh: false;
			species bed_obj refresh: false;
			species window refresh: false;
			species toilet_obj refresh: false;
			species sopha_obj refresh: false;
			species bath_obj refresh: false;
			species door refresh: false;
			species occupant ; 

			graphics "hour" {
				point loc <- {-3,-3};
				draw clock_normal size: 5 at:loc ;
				draw clock_big_hand rotate: current_date.minute*(360/60)  + 90  size: {5,5/14} at:loc + {0,0,0.1}; 
				draw clock_small_hand rotate: current_date.hour*(360/12)  + 90  size: {3,3/10} at:loc + {0,0,0.1}; 
			}
			
			graphics "Temperature"  {         
                draw "Temperature: "+(temperature with_precision(2)) size:5 at: {-7,2} color: # red font: font("SansSerif", 20, #bold);
				draw "Humidity: "+humidity size:5 at: {-7,3} color: # red font: font("SansSerif", 20, #bold);
				draw "House Temperature: "+(building[0].temp_int with_precision(2)) size:5 at: {-7,4} color: #green font: font("SansSerif", 20, #bold);
				draw "Date : "+ current_date at:{ -7,5 } color:#blue font:font("SansSerif", 20, #bold);
            }
			
		}

		display indic_occ type: java2D  {
			chart "Indicators" type: radar x_serie_labels: ["energy", "fullness", "cleanliness", "toilet", "wash clothes", "fresh air", "house cleanliness","smoke"] series_label_position: xaxis 	{
				data occupant[0].name value: [G_energy[0], G_fullness[0], G_cleanliness[0], G_toilet[0], G_wash_clothes[0],G_fresh_air[0],G_house_cleanliness[0],G_smoke[0]] color: # orange;
				data occupant[1].name value: [G_energy[1], G_fullness[1], G_cleanliness[1], G_toilet[1], G_wash_clothes[1],G_fresh_air[1],G_house_cleanliness[1],G_smoke[1]] color: # blue;
				data occupant[2].name value: [G_energy[2], G_fullness[2], G_cleanliness[2], G_toilet[2], G_wash_clothes[2],G_fresh_air[2],G_house_cleanliness[2],G_smoke[2]] color: # pink;
			}
		}
				

		display activity {
			chart name: 'Activity: Sleep:black, Relax:pink,  Shower:blue, WC:brown, Eat:violet, Cook:purple, Work:red, Smoke:greay, Wash clothes:turquoise, Go out:green, Clean house:orange, Discuss comfort:yellow, Discuss GO: gold, Modify temp:dark green' type: series  size: {1.0, 0.4} 
			position: {0, 0} x_serie_labels:(current_date) x_range: 30 y_tick_unit:1 {

			data occupant[0].name value:occupant[0].number line_visible: false color:occupant[0].my_activity_color marker: true marker_size:1 ;
			data occupant[1].name value:occupant[1].number line_visible: false color:occupant[1].my_activity_color marker: true marker_size:1 ;
			data occupant[2].name value:occupant[2].number line_visible: false color:occupant[2].my_activity_color marker: true marker_size:1 ;
			} 
		}
		
		display temperature{
			chart name: 'Temperature' type: series  size: {1.0, 0.8} 
			position: {0, 0} x_serie_labels:(current_date) x_range: 30 y_tick_unit:1 {
				data 'Outside Temperature' value:(temperature with_precision(2)) line_visible:true color: # green;
				data 'Inside Temperature' value:(building[0].temp_int with_precision(2)) line_visible:true color: #blue;
				data 'Regulation' value:building[0].thermostat*20 line_visible:true color: #red;
				data 'Comfort low'+occupant[1].name value:occupant[1].my_temp_up line_visible:true color: #yellow;
				data 'Comfort up'+occupant[1].name value:occupant[1].my_temp_low line_visible:true color: #yellow;
				data 'Incomfort up '+occupant[1].name value:(occupant[1].my_temp_up+occupant[1].delta_temp_warm) line_visible:true color: #yellow;
				data 'Incomfort low'+occupant[1].name value:(occupant[1].my_temp_low-occupant[1].delta_temp_cold)  line_visible:true color: #yellow;
			} 
		}
			
	}
}
	
experiment NoView type: batch repeat: 1 keep_seed: true until: (current_date.year > 2018 ) {
}


//**************************************END OF THE EXPERIMENTS **********************************


